"""
Provides a caching remote template loader.
"""

import os
import fcntl
import socket
import urllib
import tempfile
import datetime

from django.conf import settings
from django.template import TemplateDoesNotExist
from django.utils._os import safe_join

# Cache directory
template_cache_dir = getattr(settings, 'REMOTE_TEMPLATE_CACHE_DIR',
                             tempfile.mkdtemp())
if not os.path.exists(template_cache_dir):
    os.mkdir(template_cache_dir)

# Cache age, default 1 day
template_cache_age = datetime.timedelta(
    seconds=getattr(settings, 'REMOTE_TEMPLATE_CACHE_AGE', 60 * 60 * 24))

# At compile time, cache the URIs to search.
remote_template_uris = {}
for template_name, remote_uri in getattr(settings, 'REMOTE_TEMPLATES', []):
    cache_file = os.path.join(template_cache_dir, template_name)

    if os.path.exists(cache_file):
        # If the file exists in the cache directory, use it's modified time
        # to determine the appropriate expiration time
        mtime = os.stat(cache_file).st_mtime
        modified_date = datetime.datetime.fromtimestamp(mtime)
        exp_time = modified_date + template_cache_age
    else:
        # If the file doesn't exist, treat it as expired
        exp_time = datetime.datetime.now()

    remote_template_uris[template_name] = {
        'template_name': template_name,
        'remote_uri': remote_uri,
        'cache_file': cache_file,
        'expires': exp_time,
    }


def load_remote_template(template_name, template_dirs=None):
    'Called by Django to retrieve the content of a template'
    
    # Abort early if there is no defined mapping of remote templates
    if not remote_template_uris:
        raise TemplateDoesNotExist(
            "Your REMOTE_TEMPLATES setting is empty. " +
            "Change it to point to at least one remote template.")

    if template_name in remote_template_uris:
        rtemplate = remote_template_uris[template_name]
        try:
            # If the cached version has expired, we need to download 
            # it again
            if rtemplate['expires'] <= datetime.datetime.now():

                # Make sure tha directory structure needed to cache
                # this template is in place
                cache_file_dir = os.path.dirname(rtemplate['cache_file'])
                if not os.path.exists(cache_file_dir):
                    os.mkdir(cache_file_dir)

                # Grab the current cached content for later comparison
                if os.path.exists(rtemplate['cache_file']):
                    cache = open(rtemplate['cache_file'], 'r')
                    old_content = cache.read()
                    cache.close()
                else:
                    old_content = ''

                try:
                    # Lock the cache file to make sure no other process
                    # will try to write to it at the same time
                    cache = open(rtemplate['cache_file'], 'w')
                    fcntl.flock(cache, fcntl.LOCK_EX | fcntl.LOCK_NB)
                except IOError, e:
                    # Failed to obtain write lock, abort
                    try:
                        # catch exception if it can't close
                        cache.close()
                    except Exception:
                        pass

                    # If a lock can't be obtained, some other process is
                    # in the process of updating, serve the old content
                    # instead.
                    return (old_content, rtemplate['cache_file'])

                try:
                    # Need to set a socket timeout, otherwise urllib will
                    # block indefinitely if the remote service can't be
                    # contacted
                    old_timeout = socket.getdefaulttimeout()
                    socket.setdefaulttimeout(20)
                    remote_file = urllib.urlopen(rtemplate['remote_uri'])
                    
                    # Grab the new content from the connection
                    new_content = remote_file.read()
                except Exception, e:
                    # Failed to connect to the remote URL, fall back to old
                    # content if we have it.
                    if old_content is not None and old_content != '':
                        return (old_content, rtemplate['cache_file'])
                    # otherwise let the error propogate
                    else:
                        raise e

                if new_content != old_content:
                    # TODO: send email if they're different
                    pass

                # Save downloaded content into the cache file
                cache.write(new_content)
                
                # Release the write lock
                fcntl.flock(cache, fcntl.LOCK_UN)
                cache.close()

                # Update the expiration date on the cached file
                rtemplate['expires'] = datetime.datetime.now() + \
                                        template_cache_age

                return (new_content, rtemplate['cache_file'])

        except Exception:
            # TODO: send error email and continue
            raise TemplateDoesNotExist(
                                    'Tried %s' % rtemplate['remote_uri'])

        # Cache has not expired, service the local copy
        try:
            file = open(rtemplate['cache_file'])
            try:
                return (
                    file.read().decode(settings.FILE_CHARSET),
                    rtemplate['cache_file'])
            finally:
                file.close()
        except IOError:
            raise TemplateDoesNotExist('Tried %s' %
                                        rtemplate['cache_file'])
    else:
        raise TemplateDoesNotExist()
load_remote_template.is_usable = True
