import os
import unittest
import datetime
from tempfile import mkdtemp
from shutil import rmtree

from django.test import TestCase
from django.conf import settings

from isd_common.template_loaders import remote

class RemoteTemplateLoaderTests(TestCase):
    urls = 'isd_common.tests.urls'

    def setUp(self):
        self.cache_dir = mkdtemp()
        self.old_cache_dir = remote.template_cache_dir
        self.old_cache_age = remote.template_cache_age
        self.old_template_map = getattr(remote, 'remote_template_uris', {})
        self.old_template_dirs = settings.TEMPLATE_DIRS
        
        remote.template_cache_dir = self.cache_dir
        remote.template_cache_age = datetime.timedelta(seconds=60)  # 1 minute
        self.template = os.path.join(os.path.dirname(__file__), 'test_remote_template.html')

        remote.remote_template_uris['remote_template/test.html'] = {
            'template_name': 'remote_template/test.html',
            'remote_uri': 'file://%s' % self.template,
            'cache_file': os.path.join(self.cache_dir, 'remote_template/test.html'),
            'expires': datetime.datetime.now(),
        }
        settings.TEMPLATE_DIRS = list(settings.TEMPLATE_DIRS)
        settings.TEMPLATE_DIRS.append(os.path.dirname(__file__))

    def tearDown(self):
        remote.template_cache_dir = self.old_cache_dir
        remote.template_cache_age = self.old_cache_age
        rmtree(self.cache_dir)
        remote.remote_template_uris = self.old_template_map

    def test_load_remote_template(self):
        response = self.client.get('/remote_template/consumer/')
        self.assertContains(response, 'main-menu')

    def test_use_cached_template(self):
        # prime the cache
        response = self.client.get('/remote_template/consumer/')
        self.assertContains(response, 'main-menu')

        # modify cached file
        rtemplate = remote.remote_template_uris['remote_template/test.html']
        fh = open(rtemplate['cache_file'], 'wa')
        fh.write('\ncached-file\n')
        fh.close()

        response = self.client.get('/remote_template/consumer/')
        self.assertContains(response, 'cached-file')

    def test_reload_expired_template(self):
        # prime the cache
        response = self.client.get('/remote_template/consumer/')
        self.assertContains(response, 'main-menu')

        # modify cached file
        rtemplate = remote.remote_template_uris['remote_template/test.html']
        fh = open(rtemplate['cache_file'], 'wa')
        fh.write('\ncached-file\n')
        fh.close()

        # mark cached file as expired
        rtemplate['expires'] = datetime.datetime.now()
        
        response = self.client.get('/remote_template/consumer/')
        self.assertContains(response, 'main-menu')

    def test_remote_unavailable(self):
        # prime the cache
        response = self.client.get('/remote_template/consumer/')
        self.assertContains(response, 'main-menu')

        # modify cached file
        rtemplate = remote.remote_template_uris['remote_template/test.html']
        fh = open(rtemplate['cache_file'], 'wa')
        fh.write('\ncached-file\n')
        fh.close()

        # mark cached file as expired
        rtemplate['expires'] = datetime.datetime.now()
        
        # point to non-existant remote template
        rtemplate['remote_uri'] = 'http://localhost:9999/does/not/exist/'
        
        response = self.client.get('/remote_template/consumer/')
        self.assertContains(response, 'cached-file')

    def test_include_remote_template(self):
        response = self.client.get('/remote_template/include/')
        self.assertContains(response, '<!-- Include remote template -->')
        self.assertContains(response, 'main-menu')

def suite():
    return unittest.TestLoader().loadTestsFromName(__name__)
