import os
import unittest
import datetime
import decimal
from tempfile import mkdtemp
from shutil import rmtree

from django.core.urlresolvers import reverse
from django.test import TestCase
from django.utils.http import urlencode
from django.core.cache import cache
from django.conf import settings

from mock import Mock, patch
from contextlib import contextmanager


from isd_common.wsgi import make_app

# Original snippet from http://djangosnippets.org/snippets/2156/
class SettingDoesNotExist:
    pass


def switch_settings(**kwargs):
    """Helper method that updates settings and returns old settings."""
    old_settings = {}
    for key, new_value in kwargs.items():
        old_value = getattr(settings, key, SettingDoesNotExist)
        old_settings[key] = old_value

        if new_value is SettingDoesNotExist:
            delattr(settings, key)
        else:
            setattr(settings, key, new_value)

    return old_settings

@contextmanager
def patch_settings(**kwargs):
    old_settings = switch_settings(**kwargs)
    yield
    switch_settings(**old_settings)
# end snippet

class WsgiTests(TestCase):
    urls = 'isd_common.tests.urls'

    def setUp(self):
        """Create a temporary directory for oopses."""
        self.oops_dir = mkdtemp()

        self.environ = {
            'PATH_INFO':         '',
            'QUERY_STRING':      '',
            'REMOTE_ADDR':       '127.0.0.1',
            'REQUEST_METHOD':    'GET',
            'SCRIPT_NAME':       '',
            'SERVER_NAME':       'testserver',
            'SERVER_PORT':       '80',
            'SERVER_PROTOCOL':   'HTTP/1.1',
            'wsgi.version':      (1,0),
            'wsgi.url_scheme':   'http',
            'wsgi.multiprocess': True,
            'wsgi.multithread':  False,
            'wsgi.run_once':     False,
        }

    def tearDown(self):
        """Delete temporary oops directory."""
        rmtree(self.oops_dir)

    def request_view_exception(self, data={}):
        """Helper for creating exceptions when calling a view."""
        self.environ['PATH_INFO'] = reverse('isd_common.tests.views.cause_oops')
        if data != {}:
            self.environ['QUERY_STRING'] = urlencode(data, doseq=True)

        response = self.request_wsgi_response(self.environ)
        return response

    def request_wsgi_response(self, environ):
        start_response = Mock()
        with patch_settings(OOPS_DIR=self.oops_dir, DEBUG=False):
            app = make_app()
            response = app(environ, start_response).next()
        return response

    def read_oops_from_disk(self):
        # Helper to read in the oops written to disk.
        # There is only one date directory in the oops directory.
        dated_dirs = os.listdir(self.oops_dir)
        self.assertEqual(1, len(dated_dirs))

        # There is only one oops.
        todays_oops_path = dated_dirs[0]
        oops_list = os.listdir(self.oops_dir + '/' + todays_oops_path)
        self.assertEqual(1, len(oops_list))

        oops_path = self.oops_dir + '/' + todays_oops_path + '/' + oops_list[0]
        with open(oops_path, "r") as oops_file:
            oops_file_contents = oops_file.read()
        return oops_file_contents

    def test_oopsid_in_view_response(self):
        # The rendered response for a view exception includes the oops id.
        response = self.request_view_exception()

        self.assertTrue(self.environ['OOPSID'] in response)

    def test_traceback_in_view_oops_on_disk(self):
        # A request resulting in an exception writes the oops to disk.
        response = self.request_view_exception()

        # The oops contains the stacktrace as part of a log message.
        self.assertTrue(
            "'message': 'Traceback (most recent call last):" in (
                self.read_oops_from_disk()))

    def test_url_in_view_oops_on_disk(self):
        # A request resulting in an exception writes the oops to disk.
        response = self.request_view_exception()

        # The oops contains the url path.
        self.assertTrue(
            "URL: /wsgi/oops/" in (
                self.read_oops_from_disk()))

    def test_status_in_view_oops_on_disk(self):
        # A request resulting in an exception writes the oops to disk.
        response = self.request_view_exception()

        # The oops contains the http status code.
        self.assertTrue(
            "HTTP-Status: 500" in (
                self.read_oops_from_disk()))

def suite():
    return unittest.TestLoader().loadTestsFromName(__name__)
